package com.isu.gaswellwatch.modbus.impl;

import com.isu.gaswellwatch.dao.DeviceDao;
import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.entity.Response;
import com.isu.gaswellwatch.modbus.CommandService;
import com.isu.gaswellwatch.vo.command.Command;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Collection;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/27 0:25
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CommandServiceImpl implements CommandService {

    private final DeviceDao deviceDao;
    private final RestTemplate restTemplate = new RestTemplate();

    @Override
    public Response<String> control(Command command) {
        return this.sendCommand("control", command);
    }

    @Override
    public Response<String> collect(Command command) {
        return this.sendCommand("collect", command);
    }

    private Response<String> sendCommand(String type, Command command) {
        try {

            Collection<ModbusCommandDto> modbusCommands = Objects.requireNonNull(command).buildModbusCommand();
            if (ObjectUtils.isEmpty(modbusCommands)) {
                return Response.failed("Modbus command is empty");
            }
            this.deviceDao.getDeviceById(command.getDeviceId());

            modbusCommands.forEach(item -> {
//                .key(StringUtils.joinWith("/", identifier,
//                        MapUtil.getStr(item, "deviceId"),
//                        MapUtil.getStr(item, "commandId"),
//                        timestamp))
                item.setKey("");
                item.setIdentifier("");
            });
            RequestEntity<Collection<ModbusCommandDto>> request = RequestEntity
                    .post("http://localhost:9999/modbus-tcp/" + type)
                    .body(modbusCommands);
            ResponseEntity<Response<String>> response = this.restTemplate.exchange(request,
                    new ParameterizedTypeReference<Response<String>>() {
                    });
            if (Objects.isNull(response.getBody())) {
                return Response.failed("Nothing message");
            }
            return response.getBody();
        } catch (Exception e) {
            log.error("Send command fail", e);
            return Response.failed("Send command fail, error: " + e.getMessage());
        }
    }

}
