package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 21:02
 */
@Getter
@Setter
@SuperBuilder
public class PlungerMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 9127026055666073752L;

    /**
     * 柱塞模式初始气井状态。
     * true：开井
     * false：关井
     */
    @Builder.Default
    private int plugInitStatus = 0;

    /**
     * 柱塞模式续流时长。[0-999]:[0-59]:[0-59]
     */
    private String plugSustainTime;
    /**
     * 柱塞模式关井时长。[0-999]:[0-59]:[0-59]
     */
    private String plugCloseTime;


    public PlungerMode() {
        super(4);
        this.setCode(Command.KNPCV1_RUN_PLUNGER);
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.plugCloseTime)
                && StringUtils.isNotBlank(this.plugSustainTime)
                && this.plugInitStatus >= 0 && this.plugInitStatus <= 1;
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(150);
        //                                 0007*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     015C     0007     0E
        command.append("0110015C00070E");
        command.append("000").append(this.plugInitStatus);
        command.append(this.toHexString(this.plugSustainTime)).append(this.toHexString(this.plugCloseTime));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

    public void setPlugSustainTime(String plugSustainTime) {
        this.plugSustainTime = this.timingValidate(plugSustainTime, "plugSustainTime");
    }

    public void setPlugCloseTime(String plugCloseTime) {
        this.plugCloseTime = this.timingValidate(plugCloseTime, "plugCloseTime");
    }

}