package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;

import java.io.Serial;
import java.time.LocalTime;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 20:12
 */
@Getter
@Setter
@SuperBuilder
public class TimerMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 1612870503947873375L;

    /**
     * 定时器1使能开关
     */
    @Builder.Default
    private int timer1 = 0;
    /**
     * 定时器2使能开关
     */
    @Builder.Default
    private int timer2 = 0;
    /**
     * 定时器3使能开关
     */
    @Builder.Default
    private int timer3 = 0;
    /**
     * 定时器4使能开关
     */
    @Builder.Default
    private int timer4 = 0;

    /**
     * 定时器1开井时间
     */
    @Builder.Default
    private LocalTime timer1Open = LocalTime.MIN;
    /**
     * 定时器1关井时间
     */
    @Builder.Default
    private LocalTime timer1Close = LocalTime.MIN;

    /**
     * 定时器2开井时间
     */
    @Builder.Default
    private LocalTime timer2Open = LocalTime.MIN;
    /**
     * 定时器2关井时间
     */
    @Builder.Default
    private LocalTime timer2Close = LocalTime.MIN;

    /**
     * 定时器3开井时间
     */
    @Builder.Default
    private LocalTime timer3Open = LocalTime.MIN;
    /**
     * 定时器3关井时间
     */
    @Builder.Default
    private LocalTime timer3Close = LocalTime.MIN;

    /**
     * 定时器4开井时间
     */
    @Builder.Default
    private LocalTime timer4Open = LocalTime.MIN;
    /**
     * 定时器4关井时间
     */
    @Builder.Default
    private LocalTime timer4Close = LocalTime.MIN;

    public TimerMode() {
        super(1);
        this.setCode(Command.KNPCV1_RUN_TIMER);
    }

    @Override
    public boolean validate() {
        if (this.timer1 != 1) {
            if (Objects.isNull(this.timer1Open)) {
                this.timer1Open = LocalTime.MIN;
            }
            if (Objects.isNull(this.timer1Close)) {
                this.timer1Close = LocalTime.MIN;
            }
        }
        if (this.timer2 != 1) {
            if (Objects.isNull(this.timer2Open)) {
                this.timer2Open = LocalTime.MIN;
            }
            if (Objects.isNull(this.timer2Close)) {
                this.timer2Close = LocalTime.MIN;
            }
        }
        if (this.timer3 != 1) {
            if (Objects.isNull(this.timer3Open)) {
                this.timer3Open = LocalTime.MIN;
            }
            if (Objects.isNull(this.timer3Close)) {
                this.timer3Close = LocalTime.MIN;
            }
        }
        if (this.timer4 != 1) {
            if (Objects.isNull(this.timer4Open)) {
                this.timer4Open = LocalTime.MIN;
            }
            if (Objects.isNull(this.timer4Close)) {
                this.timer4Close = LocalTime.MIN;
            }
        }
        return super.validate()
                && this.timer1 >= 0 && this.timer1 <= 1
                && this.timer2 >= 0 && this.timer2 <= 1
                && this.timer3 >= 0 && this.timer3 <= 1
                && this.timer4 >= 0 && this.timer4 <= 1
                && Objects.nonNull(this.timer1Open) && Objects.nonNull(this.timer1Close)
                && Objects.nonNull(this.timer2Open) && Objects.nonNull(this.timer2Close)
                && Objects.nonNull(this.timer3Open) && Objects.nonNull(this.timer3Close)
                && Objects.nonNull(this.timer4Open) && Objects.nonNull(this.timer4Close);
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(150);
        //                                 001C*2->hex
        // 地址码  功能码  起始地址  连续长度   连续字长
        // 01     10     006E     001C     38
        command.append("0110006E001C38");
        command.append("000").append(this.timer1);
        command.append("000").append(this.timer2);
        command.append("000").append(this.timer3);
        command.append("000").append(this.timer4);
        command.append(this.toHexString(this.timer1Open)).append(this.toHexString(this.timer1Close))
                .append(this.toHexString(this.timer2Open)).append(this.toHexString(this.timer2Close))
                .append(this.toHexString(this.timer3Open)).append(this.toHexString(this.timer3Close))
                .append(this.toHexString(this.timer4Open)).append(this.toHexString(this.timer4Close));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

}
