package com.isu.gaswellwatch.vo.command.knpcv1.mode;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import com.isu.gaswellwatch.vo.command.Timing;
import com.isu.gaswellwatch.vo.command.knpcv1.ChangeRunMode;
import lombok.Builder;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.util.Collection;
import java.util.List;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/26 20:23
 */
@Getter
@Setter
@SuperBuilder
public class TimingMode extends ChangeRunMode implements Timing {

    @Serial
    private static final long serialVersionUID = 5065742825844725267L;

    /**
     * 计时器开井时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String timingOpen = "00:00:00";
    /**
     * 计时器关井时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String timingClose = "00:00:00";
    /**
     * 计时器延时时长。[0-999]:[0-59]:[0-59]
     */
    @Builder.Default
    private String timingCelay = "00:00:00";

    public TimingMode() {
        super(2);
        this.setCode(Command.KNPCV1_RUN_TIMING);
    }

    @Override
    public boolean validate() {
        return super.validate()
                && StringUtils.isNotBlank(this.timingOpen)
                && StringUtils.isNotBlank(this.timingClose)
                && StringUtils.isNotBlank(this.timingCelay);
    }

    public void setTimingOpen(String timingOpen) {
        this.timingOpen = this.timingValidate(timingOpen, "timingOpen");
    }

    public void setTimingClose(String timingClose) {
        this.timingClose = this.timingValidate(timingClose, "timingClose");
    }

    public void setTimingCelay(String timingCelay) {
        this.timingCelay = this.timingValidate(timingCelay, "timingCelay");
    }

    @Override
    protected Collection<ModbusCommandDto> builderSubModbusCommand() {
        StringBuilder command = new StringBuilder(60);
        //                                000A*2->hex
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0096    000A     14
        command.append("01100096000A14")
                .append(this.toHexString(this.timingOpen))
                .append(this.toHexString(this.timingClose))
                .append("0000") // 地址位：0156，值为空
                .append(this.toHexString(this.timingCelay));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

}
